%% Delta Sigma
% NTNU Project, Lars Rypestl
% Code snippets based from examples and demos in R. Schreiers Delta-Sigma
% Toolbox: http://www.mathworks.com/matlabcentral/fileexchange/19

%Working/Current directory needs to be set to the toolbox for functions to
%work!
% Specification
% Signal bandwidth 2,5MHz
% SNDR 74dB
% SFDR 84dB

clear all, close all, format compact
%% Modulator design
% Using 84dB as target for SQNR
order=3 %Mod Order
osr=32 %Oversampling Ratio
nlev=7 %Quantizer levels

zeros_opt=1 %=0; all ntf zeros at band center(DC LP mod),=1;optimizes ntf zeros,=2;zeros at band center, optimizes rest
 % OBG max
f0=0; % Mod center frequency
h_inf=1.5 %NTF magnitude, OBG

%% NTF
ntf=synthesizeNTF(order,osr,zeros_opt,h_inf,f0);
%% Spectrum
PlotExampleSpectrum(ntf,(nlev-1),osr,f0);
title('NTF Spectrum');
%% Simulate and predict SQNR
[snr_pred, amp] = predictSNR(ntf,osr);
[snr, amp]=simulateSNR(ntf,osr,amp,[],nlev,[],[],[]);
%Smoothed SQNR peak
[peak_snr2,peak_amp2] = peakSNR(snr,amp);

%Obtain SQNRs at different amplitudes
snr_copy=snr; snr_copy(snr_copy==Inf)=NaN;
[peak_snr, amp_index]=max(snr_copy);

%Amplitude with peak SQNR
Input_dB=amp(amp_index);
%Peak SQNR, SQNR at 0dB/fullscale amplitude and SQNR at -3dBFS
snr0dB=snr(23)
snr3dB=snr(20)
peak_snr, Input_dB

%Plotting SQNR vs amplitude
figure, plot(amp,snr_pred,'b',amp,snr,'gs');
grid on;
figureMagic([-100 0], 10, 1, [-40 100], 10, 1);
xlabel('Input Level, dB');
ylabel('SQNR dB');
s=sprintf('peak SNR(smoothed) = %4.1fdB\n', peak_snr2);
text(-50,80,s);
s=sprintf('SNR -3dB = %4.1fdB\n', snr3dB);
text(-50,70,s);
legend('Predicted','Simulated','Location','SouthEast')

%print -deps SNRvsInput3327



