%% Master thesis, Lars Rypestl, NTNU Spring 2011
% Noise transfer functions
% Noise gain estimation
close all, clear all, format compact

%Plot settings
P = bodeoptions; % Set phase visiblity to off and frequency units to Hz in options
P.PhaseVisible = 'off';
P.FreqUnits = 'Hz'; % Create plot with the options specified by P
P.MagScale='linear';
P.FreqScale='log';
P.xlim=[0.001 0.5];
P.ylim=[-150 20];
P.Grid='on';
fs=160e6;
[f1 f2] = ds_f1f2(32,0);
w = linspace(0.001*2*pi,2*pi*(1/(2*32)),100);

%% 3rd FB
disp('------ 3rd FB')
%Collect ABCD state-space with coefficients
[Acs,Bcs,Ccs,Dc,a,b,c,g,umax]=ABCD(3,32,2,7,'FB','CIFB',1.25e6);
H=tf(ss(Acs,Bcs(:,2),Ccs,Dc(1))); 
%EQ
NTF=1/(1-H);
sEQ=sigma(NTF,w);
rmsEQ=dbv(norm(sEQ)/sqrt(100));

%E1 transfer function
B=[b(1) 0 0]'; D=[0];
NTF_INT1=tf(ss(Acs,B,Ccs,D));
gain_INT1=NTF_INT1*NTF;
%Integrating
sE1=sigma(gain_INT1,w);
%Noise gain
rmsE1=dbv(norm(sE1)/sqrt(100))

%E2 transfer function
B=[0 c(1) 0]';
NTF_INT2=tf(ss(Acs,B,Ccs,D));
gain_INT2=NTF_INT2*NTF;
sE2=sigma(gain_INT2,w);
rmsE2=dbv(norm(sE2)/sqrt(100))

%E3 transfer function
B=[0 0 c(2)]';
NTF_INT3=tf(ss(Acs,B,Ccs,D));
gain_INT3=NTF_INT3*NTF;
sE3=sigma(gain_INT3,w);
rmsE3=dbv(norm(sE3)/sqrt(100))

%No optimization NTFQ, setting resonator to zero
Acs(2,3)=0;
H2=tf(ss(Acs,Bcs(:,2),Ccs,Dc(1))); 
NTF2=1/(1-H2);
sEQ2=sigma(NTF2,w);
rmsEQ2=dbv(norm(sEQ2)/sqrt(100))
% Plotting
figure, bodemag(NTF,gain_INT1,'--',gain_INT2,gain_INT3,P)%bodemag(NTF, NTF2,'--',P)%
%legend('NTF_{Q, Opt}','NTF_Q','Location','SouthEast')
legend('NTF_Q','STF','E2','E3','Location','SouthEast')
title('3rd order FB modulator - Noise transfer functions')
line([1/(2*32); 1/(2*32)],[50 ; -350],'LineWidth',1)
line([0.001 1/(2*32)],[rmsEQ; rmsEQ])
%line([0.001 1/(2*32)],[rmsEQ2; rmsEQ2],'LineStyle','--')
line([0.001 1/(2*32)],[rmsE3; rmsE3])
line([0.001 1/(2*32)],[rmsE2; rmsE2])
line([0.001 1/(2*32)],[rmsE1; rmsE1])
text(1/60,rmsEQ,'NG_{Q, Opt}','FontSize',8)
% text(1/60,rmsEQ2,'NG_{Q}','FontSize',8)
text(0.0011,rmsE3+4,'NG_3','FontSize',8)
text(0.0011,rmsE2+4,'NG_2','FontSize',8)
text(0.0011,rmsE1+4,'NG_{STF}','FontSize',8)

%% 3rd FF
disp('------ 3rd FF')
[Acs,Bcs,Ccs,Dc,a,b,c,g,umax]=ABCD(3,32,2,7,'FF','CIFF',1.25e6);
H=tf(ss(Acs,Bcs(:,2),Ccs,Dc(1))); 
%EQ
NTF=1/(1-H);
sEQ=sigma(NTF,w);
rmsEQ=dbv(norm(sEQ)/sqrt(100))
%E1
B=[b(1) 0 0]'; D=[0];
NTF_INT1=tf(ss(Acs,B,Ccs,D));
gain_INT1=NTF_INT1*NTF;
sE1=sigma(gain_INT1,w);
rmsE1=dbv(norm(sE1)/sqrt(100))
%E2
B=[0 c(2) 0]';
NTF_INT2=tf(ss(Acs,B,Ccs,D));
gain_INT2=NTF_INT2*NTF;
sE2=sigma(gain_INT2,w);
rmsE2=dbv(norm(sE2)/sqrt(100))
%E3
B=[0 0 c(3)]';
NTF_INT3=tf(ss(Acs,B,Ccs,D));
gain_INT3=NTF_INT3*NTF;
sE3=sigma(gain_INT3,w);
rmsE3=dbv(norm(sE3)/sqrt(100))
% Plotting
figure, bodemag(NTF,gain_INT1,'--',gain_INT2,gain_INT3,P)
legend('NTF','STF','E2','E3','Location','SouthEast')
title('3rd order FF modulator - Noise transfer functions')
line([1/(2*32); 1/(2*32)],[50 ; -350],'LineWidth',1)
%% 5th FB
disp('------ 5th FB')
[Acs,Bcs,Ccs,Dc,a,b,c,g,umax]=ABCD(5,32,1.5,2,'FB','CIFB',1.25e6);
H=tf(ss(Acs,Bcs(:,2),Ccs,Dc(1))); 
%EQ
NTF=1/(1-H);
sEQ=sigma(NTF,w);
rmsEQ=dbv(norm(sEQ)/sqrt(100))
%E1
B=[b(1) 0 0 0 0]'; D=[0];
NTF_INT1=tf(ss(Acs,B,Ccs,D));
gain_INT1=NTF_INT1*NTF;
sE1=sigma(gain_INT1,w);
rmsE1=dbv(norm(sE1)/sqrt(100))
%E2
B=[0 c(1) 0 0 0]';
NTF_INT2=tf(ss(Acs,B,Ccs,D));
gain_INT2=NTF_INT2*NTF;
sE2=sigma(gain_INT2,w);
rmsE2=dbv(norm(sE2)/sqrt(100))
%E3
B=[0 0 c(2) 0 0]';
NTF_INT3=tf(ss(Acs,B,Ccs,D));
gain_INT3=NTF_INT3*NTF;
sE3=sigma(gain_INT3,w);
rmsE3=dbv(norm(sE3)/sqrt(100))
%E4
B=[0 0 0 c(3) 0]';
NTF_INT4=tf(ss(Acs,B,Ccs,D));
gain_INT4=NTF_INT4*NTF;
sE4=sigma(gain_INT4,w);
rmsE4=dbv(norm(sE4)/sqrt(100))
%E5
B=[0 0 0 0 c(4)]';
NTF_INT5=tf(ss(Acs,B,Ccs,D));
gain_INT5=NTF_INT5*NTF;
sE5=sigma(gain_INT5,w);
rmsE5=dbv(norm(sE5)/sqrt(100))
% Plotting
figure, bodemag(NTF,gain_INT1,'--',gain_INT2,gain_INT3,gain_INT4,gain_INT5,P)
legend('NTF','STF','E2','E3','E4','E5','Location','SouthEast')
title('5th order FB modulator - Noise transfer functions')
line([1/(2*32); 1/(2*32)],[50 ; -350],'LineWidth',1)

%% 5th FF
disp('------ 5th FF')
[Acs,Bcs,Ccs,Dc,a,b,c,g,umax]=ABCD(5,32,1.5,2,'FF','CIFF',1.25e6);
H=tf(ss(Acs,Bcs(:,2),Ccs,Dc(1))); 
%EQ
NTF=1/(1-H);
sEQ=sigma(NTF,w);
rmsEQ=dbv(norm(sEQ)/sqrt(100))
%E1
B=[b(1) 0 0 0 0]'; D=[0];
NTF_INT1=tf(ss(Acs,B,Ccs,D));
gain_INT1=NTF_INT1*NTF;
sE1=sigma(gain_INT1,w);
rmsE1=dbv(norm(sE1)/sqrt(100))
%E2
B=[0 c(2) 0 0 0]';
NTF_INT2=tf(ss(Acs,B,Ccs,D));
gain_INT2=NTF_INT2*NTF;
sE2=sigma(gain_INT2,w);
rmsE2=dbv(norm(sE2)/sqrt(100))
%E3
B=[0 0 c(3) 0 0]';
NTF_INT3=tf(ss(Acs,B,Ccs,D));
gain_INT3=NTF_INT3*NTF;
sE3=sigma(gain_INT3,w);
rmsE3=dbv(norm(sE3)/sqrt(100))
%E4
B=[0 0 0 c(4) 0]';
NTF_INT4=tf(ss(Acs,B,Ccs,D));
gain_INT4=NTF_INT4*NTF;
sE4=sigma(gain_INT4,w);
rmsE4=dbv(norm(sE4)/sqrt(100))
%E5
B=[0 0 0 0 c(5)]';
NTF_INT5=tf(ss(Acs,B,Ccs,D));
gain_INT5=NTF_INT5*NTF;
sE5=sigma(gain_INT5,w);
rmsE5=dbv(norm(sE5)/sqrt(100))
% Plotting
figure, bodemag(NTF,gain_INT1,'--',gain_INT2,gain_INT3,gain_INT4,gain_INT5,P)
legend('NTF','STF','E2','E3','E4','E5','Location','SouthEast')
title('5th order FF modulator - Noise transfer functions')
line([1/(2*32); 1/(2*32)],[50 ; -350],'LineWidth',1)