%% Delta Sigma modulator
% NTNU Thesis, Lars Rypestl, 2011
% Code snippets based from examples and demos in R. Schreiers Delta-Sigma
% Toolbox: http://www.mathworks.com/matlabcentral/fileexchange/19
%Working/Current directory needs to be set to the toolbox for functions to
%work!
clear all%, close all, format compact;
%% Design parameters
order=3
osr=32
nlev=7 %Quantizer levels
h_inf=2 %max OBG
zeros_opt=1; %=0; optimizes ntf zeros
f0=0; % Mod center frequency
% Parameters for the continuous-time implementation
tdac = [0 1]	% DAC timing. [0 1] means zero-delay non-return-to-zero
form = 'FF'
typ='CIFF'
%% Synthesis
%% Noise transfer function
ntf=synthesizeNTF(order,osr,zeros_opt,h_inf,f0);
%% Simulation Input signal
% N=8192; fB=ceil(N/(2*osr)); %ftest=floor(2/3*fB);
% f=floor(fB*2/3); u=0.5*(nlev-1)*sin(2*pi*f/N*[0:N-1]);
% v=simulateDSM(u, ntf, nlev);
% %Time domain Frequency domain
% figure
% PlotExampleSpectrum(ntf,(nlev-1),osr,f0);
% title('NTF spectrum');
%% CT Realization
[ABCDc,tdac2]=realizeNTF_ct(ntf, form, tdac);
x_ampl_inn_max=0.9%[]; %Fullscale 1.8V
test_freq=1210953/160002048; % Test sinusoid frequency, normalized
%% Coefficients
[a_unscaled,g_unscaled,b_unscaled,c_unscaled]=mapABCD(ABCDc, typ);
[Ac Bc Cc Dc] = partitionABCD(ABCDc);
%Loop filter desciption
LFc = ss(Ac, Bc, Cc, Dc);
L0c = zpk(ss(Ac,Bc(:,1),Cc,Dc(1)));
%% Map CT to DT equivalent
[LF,Gp] = mapCtoD(LFc,tdac2);
ABCD = [LF.a LF.b; LF.c LF.d];
%Scale discrete ABCD
[ABCDs,umax2,S] = scaleABCD(ABCD,nlev,test_freq,1,[],x_ampl_inn_max,[]);
S = S(1:order,1:order);	% Don't worry about the extra states used in the d-t model
Acs=S*Ac/S; Bcs=S*Bc;  Ccs=Cc/S;
ABCDcs = [Acs Bcs; Ccs Dc];
%Obtain scaled coefficients
[a_scaled,g_scaled,b_scaled,c_scaled]=mapABCD(ABCDcs, typ);
%% Check realized NTF with original NTF
ntf_real=calculateTF(ABCD);
%Calculate and plot STF and NTF
bins=1200;
fs=bins/2;
f = linspace(0,2,bins);
STF_dB=dbv(evalTFP(L0c,ntf_real,f));
z = exp(2i*pi*f);
NTF_dB = dbv(evalTF(ntf_real,z));
[f1 f2] = ds_f1f2(osr,f0);
NG0 = dbv(rmsGain(ntf,f1,f2));
plot(f,STF_dB,'r'), figureMagic([0 2], 0.1, 1, [-150 10], 10, 1);
ylabel('STF, dB'), xlabel('Normalized frequency (1\rightarrow f_s)')
legend('STF_{FB}', 'STF_{FF}')%,'NTF')
grid on;
hold on;
%plot([f1 f2],NG0*[1 1],'k','Linewidth', 1);
%text( 0.15, NG0+5, sprintf('rms gain = %5.0fdB',NG0));
set(gcf,'name','STF')%'NTF and STF')
hold on
%Obtain STF attuation at fs-fb
[STF_peak findex]=max(STF_dB);
STF_peak
frequency=findex/bins*2*osr*5
Attenuation_fs_fb=dbv(evalTFP(L0c,ntf_real,(1-1/(2*osr))))
Attenuation_2fs_fb=dbv(evalTFP(L0c,ntf_real,(2-1/(2*osr))))
%% Performance
% [snr, amp]=simulateSNR(ntf,osr,amp,[],nlev,[],[],[]);
% %Smoothed SQNR peak
% [peak_snr2,peak_amp2] = peakSNR(snr,amp);
% %Obtain SQNRs at different amplitudes
% snr_copy=snr; snr_copy(snr_copy==Inf)=NaN;
% [peak_snr, amp_index]=max(snr_copy);
% Input_dB=amp(amp_index);
% %Peak SQNR, SQNR at 0dB/fullscale amplitude and SQNR at -3dBFS
% snr0dB=snr(23)
% snr3dB=snr(20)
% peak_snr, Input_dB
% 
% %Plotting SQNR vs amplitude
% figure, plot(amp,snr), grid on, figureMagic([-100 0], 10, 1, [-40 100], 10, 1);
% xlabel('Input Level, dB'),ylabel('SQNR dB');
% s=sprintf('peak SNR(smoothed) = %4.1fdB\n', peak_snr2);
% text(-50,80,s), s=sprintf('SNR -3dB = %4.1fdB\n', snr3dB);
% text(-50,70,s), legend('Predicted','Simulated','Location','SouthEast')
% % Verify that the sampled pulse response of the CT loop filter
% % matches the impulse response of the DT
% figure
% n_imp = 10;
% y = -impL1(ntf,n_imp); % Negate impL1 to make the plot prettier
% lollipop(0:n_imp,y);
% hold on;
% yl = floor(0.9*max(y)); plot([0.5 2], [yl yl], 'b', [0.5 2], [yl yl], 'bo');
% text(2,yl,'   Discrete-Time');
% grid on;
% dt = 1/16;
% yy = -pulse(LFc,[0 0;tdac],dt,n_imp);
% t = 0:dt:n_imp;
% plot(t,yy,'g');
% yl = floor(0.7*max(y)); plot([0.5 2], [yl yl], 'g');
% text(2,yl,'  Continuous-Time');
% title('Loop filter pulse/impulse responses (negated)');
% xlabel('n')
